set surface_id 1
set lines_list1 [list 1 3 4]
set lines_list2 [list 5 6]
set num_divisions [list 2 4 8 16 32 64]
set elems [list dkt drill_rot 6N Quadrilateral]
set quadratic_type [list 0 1]

set check_points ""
lappend check_points {48.0 52.0 0.0 "Disp Y" {"Contour fill" Static Displacements "" Disp_Y} {Last timestep} "" ""}
lappend check_points {48.0 52.0 0.0 "Stress Top Sy" {"Contour fill" Static Shells//Stresses_Top "" Sy} {Last timestep} "" ""}

set script_path [ file dirname [ file normalize [ info script ] ] ]
set f [open [file join $script_path "results.txt"] w]

puts -nonewline "\"Num nodes\""
foreach elem $elems {
    foreach quad_type $quadratic_type {
	if { $quad_type == 1 && $elem eq "6N" } {
	    continue
	} else {
	    if { $quad_type == 1 } {
		set qt "Quadratic"
	    } else {
		set qt "Linear"
	    }
	}
	foreach cp $check_points {
	    puts -nonewline $f "\t\"$qt $elem\""
	}
    }
}

set err [catch {
	foreach num $num_divisions {
	    
	    puts -nonewline $f "\n[expr ($num+1)*($num+1)]"
	    
	    foreach elem_type $elems {
		
		foreach quad_type $quadratic_type {
		    if { $quad_type == 1 && $elem_type eq "6N" } {
		        continue
		    } else {
		        if { $quad_type == 1 } {
		            set qt "Quadratic"
		        } else {
		            set qt "Linear"
		        }
		    }
		    
		    # Set element type
		    if { $elem_type ne "Quadrilateral" } {
		        GiD_Process Mescape Meshing ElemType Triangle $surface_id
		    } else {
		        set elem_type dkt; # This is to ensure triangular internal element type is not 6N which is incompatible with quadratic elements
		        GiD_Process Mescape Meshing ElemType Quadrilateral $surface_id
		    }    
		    gid_groups_conds::setAttributesF {blockdata[@n='General data']/container[@n='Analysis']/value[@n='triang_internal_elem']} [list v $elem_type]

		    if { $elem_type eq "6N" || $qt eq "Quadratic" } {
		        # For 6Noded triangles divide by 2 the number of divisions to obtain the same number of nodes
		        set N [expr $num/2]
		    } else {
		        set N $num
		    }
		    
		    # Run for all mesh sizes
		    GidUtils::Disable windows
		    GidUtils::DisableGraphics
		    GiD_Process Mescape Utilities Variables Model(QuadraticType) $quad_type Mescape
		    GiD_Process Mescape Meshing Structured Surfaces $surface_id escape $N {*}$lines_list1 escape 0 Mescape
		        GiD_Process Mescape Meshing Structured Surfaces $surface_id escape [expr $N/2] {*}$lines_list2 escape 0 Mescape
		    if { [lindex [GiD_Info Mesh] 0] } {
		        GiD_Process Mescape Meshing CancelMesh Yes
		    }
		    GiD_Process Mescape Meshing Generate $num MeshingParametersFrom=Model Mescape
		    set err [catch {GiD_Process Mescape Utilities Calculate} errstring]
		    after 5000
		    while 1 {
		        set modelName [lindex [lindex $::RunProcInfo 0] 0]
		        set uid [lindex [lindex $::RunProcInfo 0] 2]
		        if { $uid == "" } { break }
		        after 1000
		        update
		    }
		    GidUtils::Enable windows
		    GidUtils::EnableGraphics
		    
		    # Postprocess
		    GiD_Process Mescape Postprocess
		    foreach cp $check_points {
		        lassign $cp x y z desc result result_type value max_error
		        draw_post::set_active_result $result ""
		        lassign [draw_post::give_point_information [list $x $y $z]] mesh_id num_ nodes nodes_coords p1 min_angle results GPdata    
		        puts -nonewline $f \t[format "%.4f" "[expr abs($results)]"]
		        flush $f
		    }
		    draw_post::end_postprocess stop
		}
	    }
	}
    } errstring]

if { $err } {
    set ferr [open [file join $script_path "script.err"] w]
    puts $ferr "Script execution error: $errstring"
    close $ferr
}

flush $f
close $f
